#ifndef lint
static char	*sccsid = "@(#)$Header: help.c,v 1.8 90/01/15 23:18:58 sob Exp $";
#endif

#include "common.h"

/*
 * HELP
 *
 * Provide a naive user with a brief help message.
 *
 */

help(argc, argv)
int	argc;
char	*argv[];
{
	printf("%d This server accepts the following commands:\r\n", INF_HELP);
	printf("ARTICLE     BODY         GROUP\r\n");
	printf("HEAD        LAST         LIST\r\n");
	printf("NEXT        POST         QUIT\r\n");
	printf("STAT        NEWGROUPS    HELP\r\n");
	printf("IHAVE       NEWNEWS      SLAVE\r\n");
#if defined(XHDR) || defined(XTHREAD)
	printf("\r\nAdditionally, the following extentions are supported:\r\n\r\n");
# ifdef	XHDR
	printf("XHDR        Retrieve a single header line from a range of articles.\r\n");
# endif	XHDR
# ifdef	XTHREAD
	printf("XTHREAD     Retrieve trn thread file for the current group.\r\n");
# endif
#endif
	printf("\r\n");
	printf("Bugs to Stan Barber (Internet: nntp@tmc.edu; UUCP: ...!bcm!nntp)\r\n");
	printf(".\r\n");
	(void) fflush(stdout);
}
